@@ -1,5 +1,6 @@
 import pathlib
 import time
+from functools import lru_cache
 
 import pandas as pd
 
@@ -23,6 +24,7 @@ def start_end_2_name(start, end):
     return f'{start}{START_SEP_END}{end}'
 
 
+@lru_cache(maxsize=4)
 def files_to_dataframe(path, suffix=FILE_SUFFIX):
     """目录中文件转DataFrame
 
@@ -39,6 +41,7 @@ def files_to_dataframe(path, suffix=FILE_SUFFIX):
     pd.DataFrame
 
     """
+    path = pathlib.Path(path)
     files = list(path.glob(f'*{suffix}'))
     df = pd.DataFrame([f.name.split('.')[0].split(START_SEP_END) for f in files], columns=['start', 'end'])
     # 结束时间
