@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 # @Author     :wukan
-# @License    :(C) Copyright 2022, redhorse
+# @License    :(C) Copyright 2022
 # @Date       :2022-02-14
 
 """
@@ -11,11 +11,13 @@ API接口数据下载工具
 
 
 """
-
+import logging
 import pathlib
+from typing import List
 
 import pandas as pd
 from loguru import logger
+from tenacity import retry, before_sleep_log, wait_random
 
 from .common import FILE_SUFFIX
 from ..api.common import (
@@ -23,7 +25,15 @@ from ..api.common import (
     start_end_2_name,
     filter_range_in_dataframe,
     timeout_mtime)
-from ..common import START_SEP_END
+from ..common import START_SEP_END, get_key, read_obj, write_obj, remove_obj, TEMP_SUFFIX
+
+
+def func_pre_save(df, **kwargs):
+    return df
+
+
+def func_post_download(df, **kwargs):
+    return df
 
 
 class Dump:
@@ -34,7 +44,7 @@ class Dump:
     # 文件夹中的所有文件，用于判断文件是否已经存在
     files_df = None
     # 下载的单期数据
-    df = None
+    dfs = {}
     # 底层调用的函数名
     func_name = None
     # 位置参数
@@ -42,7 +52,7 @@ class Dump:
     # 命名参数
     kwargs = {}
 
-    def __init__(self, api, path, file_names):
+    def __init__(self, api, path: str, file_names: List[str]):
         """初始化
 
         Parameters
@@ -62,14 +72,9 @@ class Dump:
 
     def reset(self):
         """重置"""
-        self.file_path = None
-        self.files_df = None
-        self.df = None
-        self.func_name = None
-        self.args = ()
-        self.kwargs = {}
+        self.dfs = {}
 
-    def set_parameters(self, func_name, *args, **kwargs):
+    def set_parameters(self, func_name: str, *args, **kwargs) -> None:
         """设置查询参数
 
         不同API不同，需要定制重载
@@ -94,7 +99,7 @@ class Dump:
             self.file_path = None
         pass
 
-    def exists(self, timeout):
+    def exists(self, timeout: int) -> bool:
         """检查文件是否存在，防止重复下载
 
         Parameters
@@ -124,15 +129,21 @@ class Dump:
             else:
                 return False
 
-    def download(self, kw=None):
+    @retry(wait=wait_random(10, 20), before_sleep=before_sleep_log(logger, logging.DEBUG))
+    async def download(self, use_await: bool, kw: List[str], post_download=func_post_download, post_download_kwargs={}):
         """下载动作。每个API的函数与参数不同，需定制重载
 
+        Parameters
+        ----------
+        use_await: bool
+            所调用的API是同步函数还是异步函数
+        kw:
+            需要传送的参数
+
         Returns
         -------
         pd.DataFrame
             查询的数据
-        kw:
-            需要传送的参数。None表示全传
 
         Notes
         -----
@@ -141,54 +152,68 @@ class Dump:
 
         """
         logger.info('下载 {} {} {}', self.func_name, self.args, self.kwargs)
-        api = getattr(self.api, self.func_name)
-        if kw is None:
-            self.df = api(*self.args, **self.kwargs)
-        else:
-            # 只有约定的键才做为参数
-            _kwargs = {k: v for k, v in self.kwargs.items() if k in kw}
-            self.df = api(*self.args, **_kwargs)
-        # 部分API返回为None
-        if self.df is None:
-            self.df = pd.DataFrame()
-        # logger.info('数据量 {} {} {} {}', len(self.df), self.func_name, self.args, self.kwargs)
-        return self.df
 
-    def save(self, save_empty, pre_save=None, pre_save_kwargs={}):
+        # 只有约定的键才做为参数
+        _kwargs = {k: v for k, v in self.kwargs.items() if k in kw}
+        # 生成hash，用于数据缓存
+        key = get_key(self.func_name, self.args, _kwargs, TEMP_SUFFIX)
+        df = read_obj(key)
+        if df is None:
+            api = getattr(self.api, self.func_name)
+            if use_await:
+                df = await api(*self.args, **_kwargs)
+            else:
+                df = api(*self.args, **_kwargs)
+            if isinstance(df, dict):
+                if df.get('status', 200) != 200:
+                    logger.error(f'{df}')
+                    raise
+            write_obj(df, key)
+            if len(df) < 30:
+                logger.warning('写入缓存 {}/{} {} {}', len(df), len(self.dfs), self.func_name, key)
+        else:
+            logger.info('命中缓存 {}/{} {} {}', len(df), len(self.dfs), self.func_name, key)
+        if df is not None:
+            df = post_download(df, **post_download_kwargs)
+            self.dfs[key] = df
+
+        return self.dfs
+
+    def save(self, pre_save=func_pre_save, pre_save_kwargs={}):
         """保存数据
 
         Parameters
         ----------
-        save_empty: bool
-            空DataFrame是否保存。全量下载前期不保存，后期得保存，防重复下载
-            读取文件夹时，只要前面的文件不为emtpy就能正常打开
         pre_save: func
             保存前的处理函数，特殊处理用
         pre_save_kwargs: dict
             保存存前处理函数的参数
 
         """
-        df = self.df
-        if df is None:
-            raise Exception('需要在download中设置数据到self.df后才能保存')
-
-        if hasattr(df, 'empty') and df.empty:
-            # 丢弃表头，防止concat时float字段被改成了object
-            df = pd.DataFrame()
-            if not save_empty:
-                return
+        dfs = [pre_save(df, **pre_save_kwargs) for key, df in self.dfs.items()]
+        if len(dfs) > 0:
+            if isinstance(dfs[0].index, pd.RangeIndex):
+                df = pd.concat(dfs, ignore_index=True)
+            else:
+                df = pd.concat(dfs)
         else:
-            if pre_save is not None:
-                df = pre_save(df, **pre_save_kwargs)
+            df = pd.DataFrame()
 
         self.path.mkdir(parents=True, exist_ok=True)
+
         # 保存
-        logger.info('保存 {} {} {}', len(self.df), self.func_name, self.file_path)
+        level = "WARNING" if len(df) < 30 else "INFO"
+        logger.log(level, '保存 {}/{} {} {}', len(df), len(dfs), self.func_name, self.file_path)
+
         try:
             df.to_parquet(self.file_path, compression='zstd')
+            for k, v in self.dfs.items():
+                remove_obj(k)
         except Exception as e:
             print(df)
             raise e
+        finally:
+            self.dfs = {}
 
     def load(self):
         """加载数据"""
